# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.telcoautomation.v1",
    manifest={
        "BlueprintView",
        "DeploymentView",
        "ResourceType",
        "Status",
        "DeploymentLevel",
        "OrchestrationCluster",
        "EdgeSlm",
        "Blueprint",
        "PublicBlueprint",
        "Deployment",
        "HydratedDeployment",
        "ListOrchestrationClustersRequest",
        "ListOrchestrationClustersResponse",
        "GetOrchestrationClusterRequest",
        "CreateOrchestrationClusterRequest",
        "DeleteOrchestrationClusterRequest",
        "ListEdgeSlmsRequest",
        "ListEdgeSlmsResponse",
        "GetEdgeSlmRequest",
        "CreateEdgeSlmRequest",
        "DeleteEdgeSlmRequest",
        "CreateBlueprintRequest",
        "UpdateBlueprintRequest",
        "GetBlueprintRequest",
        "DeleteBlueprintRequest",
        "ListBlueprintsRequest",
        "ListBlueprintsResponse",
        "ApproveBlueprintRequest",
        "ProposeBlueprintRequest",
        "RejectBlueprintRequest",
        "ListBlueprintRevisionsRequest",
        "ListBlueprintRevisionsResponse",
        "SearchBlueprintRevisionsRequest",
        "SearchBlueprintRevisionsResponse",
        "DiscardBlueprintChangesRequest",
        "DiscardBlueprintChangesResponse",
        "ListPublicBlueprintsRequest",
        "ListPublicBlueprintsResponse",
        "GetPublicBlueprintRequest",
        "CreateDeploymentRequest",
        "UpdateDeploymentRequest",
        "GetDeploymentRequest",
        "RemoveDeploymentRequest",
        "ListDeploymentsRequest",
        "ListDeploymentsResponse",
        "ListDeploymentRevisionsRequest",
        "ListDeploymentRevisionsResponse",
        "SearchDeploymentRevisionsRequest",
        "SearchDeploymentRevisionsResponse",
        "DiscardDeploymentChangesRequest",
        "DiscardDeploymentChangesResponse",
        "ApplyDeploymentRequest",
        "ComputeDeploymentStatusRequest",
        "ComputeDeploymentStatusResponse",
        "RollbackDeploymentRequest",
        "OperationMetadata",
        "GetHydratedDeploymentRequest",
        "ListHydratedDeploymentsRequest",
        "ListHydratedDeploymentsResponse",
        "UpdateHydratedDeploymentRequest",
        "ApplyHydratedDeploymentRequest",
        "ManagementConfig",
        "StandardManagementConfig",
        "FullManagementConfig",
        "MasterAuthorizedNetworksConfig",
        "File",
        "ResourceStatus",
        "NFDeployStatus",
        "NFDeploySiteStatus",
        "HydrationStatus",
        "SiteVersion",
        "WorkloadStatus",
    },
)


class BlueprintView(proto.Enum):
    r"""BlueprintView defines the type of view of the blueprint.

    Values:
        BLUEPRINT_VIEW_UNSPECIFIED (0):
            Unspecified enum value.
        BLUEPRINT_VIEW_BASIC (1):
            View which only contains metadata.
        BLUEPRINT_VIEW_FULL (2):
            View which contains metadata and files it
            encapsulates.
    """
    BLUEPRINT_VIEW_UNSPECIFIED = 0
    BLUEPRINT_VIEW_BASIC = 1
    BLUEPRINT_VIEW_FULL = 2


class DeploymentView(proto.Enum):
    r"""DeploymentView defines the type of view of the deployment.

    Values:
        DEPLOYMENT_VIEW_UNSPECIFIED (0):
            Unspecified enum value.
        DEPLOYMENT_VIEW_BASIC (1):
            View which only contains metadata.
        DEPLOYMENT_VIEW_FULL (2):
            View which contains metadata and files it
            encapsulates.
    """
    DEPLOYMENT_VIEW_UNSPECIFIED = 0
    DEPLOYMENT_VIEW_BASIC = 1
    DEPLOYMENT_VIEW_FULL = 2


class ResourceType(proto.Enum):
    r"""Represent type of CR.

    Values:
        RESOURCE_TYPE_UNSPECIFIED (0):
            Unspecified resource type.
        NF_DEPLOY_RESOURCE (1):
            User specified NF Deploy CR.
        DEPLOYMENT_RESOURCE (2):
            CRs that are part of a blueprint.
    """
    RESOURCE_TYPE_UNSPECIFIED = 0
    NF_DEPLOY_RESOURCE = 1
    DEPLOYMENT_RESOURCE = 2


class Status(proto.Enum):
    r"""Status of an entity (resource, deployment).

    Values:
        STATUS_UNSPECIFIED (0):
            Unknown state.
        STATUS_IN_PROGRESS (1):
            Under progress.
        STATUS_ACTIVE (2):
            Running and ready to serve traffic.
        STATUS_FAILED (3):
            Failed or stalled.
        STATUS_DELETING (4):
            Delete in progress.
        STATUS_DELETED (5):
            Deleted deployment.
        STATUS_PEERING (10):
            NFDeploy specific status. Peering in
            progress.
        STATUS_NOT_APPLICABLE (11):
            K8s objects such as
            NetworkAttachmentDefinition don't have a defined
            status.
    """
    STATUS_UNSPECIFIED = 0
    STATUS_IN_PROGRESS = 1
    STATUS_ACTIVE = 2
    STATUS_FAILED = 3
    STATUS_DELETING = 4
    STATUS_DELETED = 5
    STATUS_PEERING = 10
    STATUS_NOT_APPLICABLE = 11


class DeploymentLevel(proto.Enum):
    r"""DeploymentLevel of a blueprint signifies where the blueprint
    will be applied.

    Values:
        DEPLOYMENT_LEVEL_UNSPECIFIED (0):
            Default unspecified deployment level.
        HYDRATION (1):
            Blueprints at HYDRATION level cannot be used
            to create a Deployment (A user cannot manually
            initate deployment of these blueprints on
            orchestration or workload cluster).
            These blueprints stay in a user's private
            catalog and are configured and deployed by TNA
            automation.
        SINGLE_DEPLOYMENT (2):
            Blueprints at SINGLE_DEPLOYMENT level can be a) Modified in
            private catalog. b) Used to create a deployment on
            orchestration cluster by the user, once approved.
        MULTI_DEPLOYMENT (3):
            Blueprints at MULTI_DEPLOYMENT level can be a) Modified in
            private catalog. b) Used to create a deployment on
            orchestration cluster which will create further hydrated
            deployments.
        WORKLOAD_CLUSTER_DEPLOYMENT (4):
            Blueprints at WORKLOAD_CLUSTER_DEPLOYMENT level can be a)
            Modified in private catalog. b) Used to create a deployment
            on workload cluster by the user, once approved.
    """
    DEPLOYMENT_LEVEL_UNSPECIFIED = 0
    HYDRATION = 1
    SINGLE_DEPLOYMENT = 2
    MULTI_DEPLOYMENT = 3
    WORKLOAD_CLUSTER_DEPLOYMENT = 4


class OrchestrationCluster(proto.Message):
    r"""Orchestration cluster represents a GKE cluster with config
    controller and TNA specific components installed on it.

    Attributes:
        name (str):
            Name of the orchestration cluster. The name
            of orchestration cluster cannot be more than 24
            characters.
        management_config (google.cloud.telcoautomation_v1.types.ManagementConfig):
            Management configuration of the underlying
            GKE cluster.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. [Output only] Create time stamp.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. [Output only] Update time stamp.
        labels (MutableMapping[str, str]):
            Labels as key value pairs.
        tna_version (str):
            Output only. Provides the TNA version
            installed on the cluster.
        state (google.cloud.telcoautomation_v1.types.OrchestrationCluster.State):
            Output only. State of the Orchestration
            Cluster.
    """

    class State(proto.Enum):
        r"""Possible states that the Orchestration Cluster can be in.

        Values:
            STATE_UNSPECIFIED (0):
                Unspecified state.
            CREATING (1):
                OrchestrationCluster is being created.
            ACTIVE (2):
                OrchestrationCluster has been created and is
                ready for use.
            DELETING (3):
                OrchestrationCluster is being deleted.
            FAILED (4):
                OrchestrationCluster encountered an error and
                is in an indeterministic state. User can still
                initiate a delete operation on this state.
        """
        STATE_UNSPECIFIED = 0
        CREATING = 1
        ACTIVE = 2
        DELETING = 3
        FAILED = 4

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    management_config: "ManagementConfig" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="ManagementConfig",
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=4,
    )
    tna_version: str = proto.Field(
        proto.STRING,
        number=6,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=7,
        enum=State,
    )


class EdgeSlm(proto.Message):
    r"""EdgeSlm represents an SLM instance which manages the
    lifecycle of edge components installed on Workload clusters
    managed by an Orchestration Cluster.

    Attributes:
        name (str):
            Name of the EdgeSlm resource.
        orchestration_cluster (str):
            Immutable. Reference to the orchestration cluster on which
            templates for this resources will be applied. This should be
            of format
            projects/{project}/locations/{location}/orchestrationClusters/{orchestration_cluster}.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. [Output only] Create time stamp.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. [Output only] Update time stamp.
        labels (MutableMapping[str, str]):
            Optional. Labels as key value pairs. The key
            and value should contain characters which are
            UTF-8 compliant and less than 50 characters.
        tna_version (str):
            Output only. Provides the active TNA version
            for this resource.
        state (google.cloud.telcoautomation_v1.types.EdgeSlm.State):
            Output only. State of the EdgeSlm resource.
        workload_cluster_type (google.cloud.telcoautomation_v1.types.EdgeSlm.WorkloadClusterType):
            Optional. Type of workload cluster for which
            an EdgeSLM resource is created.
    """

    class State(proto.Enum):
        r"""Possible states of the resource.

        Values:
            STATE_UNSPECIFIED (0):
                Unspecified state.
            CREATING (1):
                EdgeSlm is being created.
            ACTIVE (2):
                EdgeSlm has been created and is ready for
                use.
            DELETING (3):
                EdgeSlm is being deleted.
            FAILED (4):
                EdgeSlm encountered an error and is in an
                indeterministic state. User can still initiate a
                delete operation on this state.
        """
        STATE_UNSPECIFIED = 0
        CREATING = 1
        ACTIVE = 2
        DELETING = 3
        FAILED = 4

    class WorkloadClusterType(proto.Enum):
        r"""Workload clusters supported by TNA. New values will be added
        to the enum list as TNA adds supports for new workload clusters
        in future.

        Values:
            WORKLOAD_CLUSTER_TYPE_UNSPECIFIED (0):
                Unspecified workload cluster.
            GDCE (1):
                Workload cluster is a GDCE cluster.
            GKE (2):
                Workload cluster is a GKE cluster.
        """
        WORKLOAD_CLUSTER_TYPE_UNSPECIFIED = 0
        GDCE = 1
        GKE = 2

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    orchestration_cluster: str = proto.Field(
        proto.STRING,
        number=5,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=4,
    )
    tna_version: str = proto.Field(
        proto.STRING,
        number=6,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=7,
        enum=State,
    )
    workload_cluster_type: WorkloadClusterType = proto.Field(
        proto.ENUM,
        number=8,
        enum=WorkloadClusterType,
    )


class Blueprint(proto.Message):
    r"""A Blueprint contains a collection of kubernetes resources in
    the form of YAML files. The file contents of a blueprint are
    collectively known as package. A blueprint can be
    a) imported from TNA's public catalog
    b) modified as per a user's need
    c) proposed and approved.
    On approval, a revision of blueprint is created which can be
    used to create a deployment on Orchestration or Workload
    Cluster.

    Attributes:
        name (str):
            The name of the blueprint. If unspecified, the name will be
            autogenerated from server side. Name of the blueprint must
            not contain ``@`` character.
        revision_id (str):
            Output only. Immutable. The revision ID of
            the blueprint. A new revision is committed
            whenever a blueprint is approved.
        source_blueprint (str):
            Required. Immutable. The public blueprint ID
            from which this blueprint was created.
        revision_create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The timestamp that the revision
            was created.
        approval_state (google.cloud.telcoautomation_v1.types.Blueprint.ApprovalState):
            Output only. Approval state of the blueprint
            (DRAFT, PROPOSED, APPROVED)
        display_name (str):
            Optional. Human readable name of a Blueprint.
        repository (str):
            Output only. Name of the repository where the
            blueprint files are stored.
        files (MutableSequence[google.cloud.telcoautomation_v1.types.File]):
            Optional. Files present in a blueprint.
            When invoking UpdateBlueprint API, only the
            modified files should be included in this. Files
            that are not included in the update of a
            blueprint will not be changed.
        labels (MutableMapping[str, str]):
            Optional. Labels are key-value attributes
            that can be set on a blueprint resource by the
            user.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Blueprint creation time.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The timestamp when the blueprint
            was updated.
        source_provider (str):
            Output only. Source provider is the author of
            a public blueprint, from which this blueprint is
            created.
        deployment_level (google.cloud.telcoautomation_v1.types.DeploymentLevel):
            Output only. DeploymentLevel of a blueprint signifies where
            the blueprint will be applied. e.g. [HYDRATION,
            SINGLE_DEPLOYMENT, MULTI_DEPLOYMENT]
        rollback_support (bool):
            Output only. Indicates if the deployment
            created from this blueprint can be rolled back.
    """

    class ApprovalState(proto.Enum):
        r"""Approval state indicates the state of a Blueprint in its
        approval lifecycle.

        Values:
            APPROVAL_STATE_UNSPECIFIED (0):
                Unspecified state.
            DRAFT (1):
                A blueprint starts in DRAFT state once it is
                created. All edits are made to the blueprint in
                DRAFT state.
            PROPOSED (2):
                When the edits are ready for review,
                blueprint can be proposed and moves to PROPOSED
                state. Edits cannot be made to a blueprint in
                PROPOSED state.
            APPROVED (3):
                When a proposed blueprint is approved, it
                moves to APPROVED state. A new revision is
                committed. The latest committed revision can be
                used to create a deployment on Orchestration or
                Workload Cluster. Edits to an APPROVED blueprint
                changes its state back to DRAFT. The last
                committed revision of a blueprint represents its
                latest APPROVED state.
        """
        APPROVAL_STATE_UNSPECIFIED = 0
        DRAFT = 1
        PROPOSED = 2
        APPROVED = 3

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    revision_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    source_blueprint: str = proto.Field(
        proto.STRING,
        number=3,
    )
    revision_create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    approval_state: ApprovalState = proto.Field(
        proto.ENUM,
        number=6,
        enum=ApprovalState,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=7,
    )
    repository: str = proto.Field(
        proto.STRING,
        number=8,
    )
    files: MutableSequence["File"] = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message="File",
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=10,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=11,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=12,
        message=timestamp_pb2.Timestamp,
    )
    source_provider: str = proto.Field(
        proto.STRING,
        number=13,
    )
    deployment_level: "DeploymentLevel" = proto.Field(
        proto.ENUM,
        number=14,
        enum="DeploymentLevel",
    )
    rollback_support: bool = proto.Field(
        proto.BOOL,
        number=15,
    )


class PublicBlueprint(proto.Message):
    r"""A Blueprint contains a collection of kubernetes resources in
    the form of YAML files. The file contents of a blueprint are
    collectively known as package.
    Public blueprint is a TNA provided blueprint that in present in
    TNA's public catalog. A user can copy the public blueprint to
    their private catalog for further modifications.

    Attributes:
        name (str):
            Name of the public blueprint.
        display_name (str):
            The display name of the public blueprint.
        description (str):
            The description of the public blueprint.
        deployment_level (google.cloud.telcoautomation_v1.types.DeploymentLevel):
            DeploymentLevel of a blueprint signifies where the blueprint
            will be applied. e.g. [HYDRATION, SINGLE_DEPLOYMENT,
            MULTI_DEPLOYMENT]
        source_provider (str):
            Source provider is the author of a public
            blueprint. e.g. Google, vendors
        rollback_support (bool):
            Output only. Indicates if the deployment
            created from this blueprint can be rolled back.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    deployment_level: "DeploymentLevel" = proto.Field(
        proto.ENUM,
        number=4,
        enum="DeploymentLevel",
    )
    source_provider: str = proto.Field(
        proto.STRING,
        number=5,
    )
    rollback_support: bool = proto.Field(
        proto.BOOL,
        number=15,
    )


class Deployment(proto.Message):
    r"""Deployment contains a collection of YAML files (This
    collection is also known as package) that can to applied on an
    orchestration cluster (GKE cluster with TNA addons) or a
    workload cluster.

    Attributes:
        name (str):
            The name of the deployment.
        revision_id (str):
            Output only. Immutable. The revision ID of
            the deployment. A new revision is committed
            whenever a change in deployment is applied.
        source_blueprint_revision (str):
            Required. The blueprint revision from which
            this deployment was created.
        revision_create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The timestamp that the revision
            was created.
        state (google.cloud.telcoautomation_v1.types.Deployment.State):
            Output only. State of the deployment (DRAFT,
            APPLIED, DELETING).
        display_name (str):
            Optional. Human readable name of a
            Deployment.
        repository (str):
            Output only. Name of the repository where the
            deployment package files are stored.
        files (MutableSequence[google.cloud.telcoautomation_v1.types.File]):
            Optional. Files present in a deployment.
            When invoking UpdateDeployment API, only the
            modified files should be included in this. Files
            that are not included in the update of a
            deployment will not be changed.
        labels (MutableMapping[str, str]):
            Optional. Labels are key-value attributes
            that can be set on a deployment resource by the
            user.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Deployment creation time.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The timestamp when the
            deployment was updated.
        source_provider (str):
            Output only. Source provider is the author of
            a public blueprint, from which this deployment
            is created.
        workload_cluster (str):
            Optional. Immutable. The WorkloadCluster on which to create
            the Deployment. This field should only be passed when the
            deployment_level of the source blueprint specifies
            deployments on workload clusters e.g.
            WORKLOAD_CLUSTER_DEPLOYMENT.
        deployment_level (google.cloud.telcoautomation_v1.types.DeploymentLevel):
            Output only. Attributes to where the deployment can inflict
            changes. The value can only be [SINGLE_DEPLOYMENT,
            MULTI_DEPLOYMENT].
        rollback_support (bool):
            Output only. Indicates if the deployment can
            be rolled back, exported from public blueprint.
    """

    class State(proto.Enum):
        r"""State defines which state the current deployment is in.

        Values:
            STATE_UNSPECIFIED (0):
                Unspecified state.
            DRAFT (1):
                A deployment starts in DRAFT state. All edits
                are made in DRAFT state. A deployment opened for
                editing after applying will be in draft state,
                while its prevision revision will be its current
                applied version.
            APPLIED (2):
                This state means that the contents (YAML
                files containing kubernetes resources) of the
                deployment have been applied to an Orchestration
                or Workload Cluster. A revision is created when
                a deployment is applied. This revision will
                represent the latest view of what is applied on
                the cluster until the deployment is modified and
                applied again, which will create a new revision.
            DELETING (3):
                A deployment in DELETING state has been marked for deletion.
                Its deletion status can be queried using
                ``ComputeDeploymentStatus`` API. No updates are allowed to a
                deployment in DELETING state.
        """
        STATE_UNSPECIFIED = 0
        DRAFT = 1
        APPLIED = 2
        DELETING = 3

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    revision_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    source_blueprint_revision: str = proto.Field(
        proto.STRING,
        number=3,
    )
    revision_create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=5,
        enum=State,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=6,
    )
    repository: str = proto.Field(
        proto.STRING,
        number=7,
    )
    files: MutableSequence["File"] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message="File",
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=9,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=10,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=11,
        message=timestamp_pb2.Timestamp,
    )
    source_provider: str = proto.Field(
        proto.STRING,
        number=12,
    )
    workload_cluster: str = proto.Field(
        proto.STRING,
        number=13,
    )
    deployment_level: "DeploymentLevel" = proto.Field(
        proto.ENUM,
        number=14,
        enum="DeploymentLevel",
    )
    rollback_support: bool = proto.Field(
        proto.BOOL,
        number=15,
    )


class HydratedDeployment(proto.Message):
    r"""A collection of kubernetes yaml files which are deployed on a
    Workload Cluster. Hydrated Deployments are created by TNA intent
    based automation.

    Attributes:
        name (str):
            Output only. The name of the hydrated
            deployment.
        state (google.cloud.telcoautomation_v1.types.HydratedDeployment.State):
            Output only. State of the hydrated deployment
            (DRAFT, APPLIED).
        files (MutableSequence[google.cloud.telcoautomation_v1.types.File]):
            Optional. File contents of a hydrated
            deployment. When invoking
            UpdateHydratedBlueprint API, only the modified
            files should be included in this. Files that are
            not included in the update of a hydrated
            deployment will not be changed.
        workload_cluster (str):
            Output only. WorkloadCluster identifies which
            workload cluster will the hydrated deployment
            will be deployed on.
    """

    class State(proto.Enum):
        r"""State defines which state the current hydrated deployment is
        in.

        Values:
            STATE_UNSPECIFIED (0):
                Unspecified state.
            DRAFT (1):
                A hydrated deployment starts in DRAFT state.
                All edits are made in DRAFT state.
            APPLIED (2):
                When the edit is applied, the hydrated
                deployment moves to APPLIED state. No changes
                can be made once a hydrated deployment is
                applied.
        """
        STATE_UNSPECIFIED = 0
        DRAFT = 1
        APPLIED = 2

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=2,
        enum=State,
    )
    files: MutableSequence["File"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="File",
    )
    workload_cluster: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListOrchestrationClustersRequest(proto.Message):
    r"""Message for requesting list of OrchestrationClusters.

    Attributes:
        parent (str):
            Required. Parent value for
            ListOrchestrationClustersRequest
        page_size (int):
            Requested page size. Server may return fewer
            items than requested. If unspecified, server
            will pick an appropriate default.
        page_token (str):
            A token identifying a page of results the
            server should return.
        filter (str):
            Filtering results.
        order_by (str):
            Hint for how to order the results.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListOrchestrationClustersResponse(proto.Message):
    r"""Message for response to listing OrchestrationClusters.

    Attributes:
        orchestration_clusters (MutableSequence[google.cloud.telcoautomation_v1.types.OrchestrationCluster]):
            The list of OrchestrationCluster
        next_page_token (str):
            A token identifying a page of results the
            server should return.
        unreachable (MutableSequence[str]):
            Locations that could not be reached.
    """

    @property
    def raw_page(self):
        return self

    orchestration_clusters: MutableSequence[
        "OrchestrationCluster"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="OrchestrationCluster",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    unreachable: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class GetOrchestrationClusterRequest(proto.Message):
    r"""Message for getting a OrchestrationCluster.

    Attributes:
        name (str):
            Required. Name of the resource
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateOrchestrationClusterRequest(proto.Message):
    r"""Message for creating a OrchestrationCluster.

    Attributes:
        parent (str):
            Required. Value for parent.
        orchestration_cluster_id (str):
            Required. Id of the requesting object If auto-generating Id
            server-side, remove this field and orchestration_cluster_id
            from the method_signature of Create RPC
        orchestration_cluster (google.cloud.telcoautomation_v1.types.OrchestrationCluster):
            Required. The resource being created
        request_id (str):
            Optional. An optional request ID to identify
            requests. Specify a unique request ID so that if
            you must retry your request, the server will
            know to ignore the request if it has already
            been completed. The server will guarantee that
            for at least 60 minutes since the first request.

            For example, consider a situation where you make
            an initial request and the request times out. If
            you make the request again with the same request
            ID, the server can check if original operation
            with the same request ID was received, and if
            so, will ignore the second request. This
            prevents clients from accidentally creating
            duplicate commitments.

            The request ID must be a valid UUID with the
            exception that zero UUID is not supported
            (00000000-0000-0000-0000-000000000000).
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    orchestration_cluster_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    orchestration_cluster: "OrchestrationCluster" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="OrchestrationCluster",
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


class DeleteOrchestrationClusterRequest(proto.Message):
    r"""Message for deleting a OrchestrationCluster.

    Attributes:
        name (str):
            Required. Name of the resource
        request_id (str):
            Optional. An optional request ID to identify
            requests. Specify a unique request ID so that if
            you must retry your request, the server will
            know to ignore the request if it has already
            been completed. The server will guarantee that
            for at least 60 minutes after the first request.

            For example, consider a situation where you make
            an initial request and the request times out. If
            you make the request again with the same request
            ID, the server can check if original operation
            with the same request ID was received, and if
            so, will ignore the second request. This
            prevents clients from accidentally creating
            duplicate commitments.

            The request ID must be a valid UUID with the
            exception that zero UUID is not supported
            (00000000-0000-0000-0000-000000000000).
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ListEdgeSlmsRequest(proto.Message):
    r"""Message for requesting list of EdgeSlms

    Attributes:
        parent (str):
            Required. Parent value for
            ListEdgeSlmsRequest
        page_size (int):
            Requested page size. Server may return fewer
            items than requested. If unspecified, server
            will pick an appropriate default.
        page_token (str):
            A token identifying a page of results the
            server should return.
        filter (str):
            Filtering results
        order_by (str):
            Hint for how to order the results
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListEdgeSlmsResponse(proto.Message):
    r"""Message for response to listing EdgeSlms.

    Attributes:
        edge_slms (MutableSequence[google.cloud.telcoautomation_v1.types.EdgeSlm]):
            The list of EdgeSlm
        next_page_token (str):
            A token identifying a page of results the
            server should return.
        unreachable (MutableSequence[str]):
            Locations that could not be reached.
    """

    @property
    def raw_page(self):
        return self

    edge_slms: MutableSequence["EdgeSlm"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="EdgeSlm",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    unreachable: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class GetEdgeSlmRequest(proto.Message):
    r"""Message for getting a EdgeSlm.

    Attributes:
        name (str):
            Required. Name of the resource
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateEdgeSlmRequest(proto.Message):
    r"""Message for creating a EdgeSlm.

    Attributes:
        parent (str):
            Required. Value for parent.
        edge_slm_id (str):
            Required. Id of the requesting object If auto-generating Id
            server-side, remove this field and edge_slm_id from the
            method_signature of Create RPC
        edge_slm (google.cloud.telcoautomation_v1.types.EdgeSlm):
            Required. The resource being created
        request_id (str):
            Optional. An optional request ID to identify
            requests. Specify a unique request ID so that if
            you must retry your request, the server will
            know to ignore the request if it has already
            been completed. The server will guarantee that
            for at least 60 minutes since the first request.

            For example, consider a situation where you make
            an initial request and the request times out. If
            you make the request again with the same request
            ID, the server can check if original operation
            with the same request ID was received, and if
            so, will ignore the second request. This
            prevents clients from accidentally creating
            duplicate commitments.

            The request ID must be a valid UUID with the
            exception that zero UUID is not supported
            (00000000-0000-0000-0000-000000000000).
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    edge_slm_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    edge_slm: "EdgeSlm" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="EdgeSlm",
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


class DeleteEdgeSlmRequest(proto.Message):
    r"""Message for deleting a EdgeSlm.

    Attributes:
        name (str):
            Required. Name of the resource
        request_id (str):
            Optional. An optional request ID to identify
            requests. Specify a unique request ID so that if
            you must retry your request, the server will
            know to ignore the request if it has already
            been completed. The server will guarantee that
            for at least 60 minutes after the first request.

            For example, consider a situation where you make
            an initial request and the request times out. If
            you make the request again with the same request
            ID, the server can check if original operation
            with the same request ID was received, and if
            so, will ignore the second request. This
            prevents clients from accidentally creating
            duplicate commitments.

            The request ID must be a valid UUID with the
            exception that zero UUID is not supported
            (00000000-0000-0000-0000-000000000000).
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CreateBlueprintRequest(proto.Message):
    r"""Request object for ``CreateBlueprint``.

    Attributes:
        parent (str):
            Required. The name of parent resource. Format should be -
            "projects/{project_id}/locations/{location_name}/orchestrationClusters/{orchestration_cluster}".
        blueprint_id (str):
            Optional. The name of the blueprint.
        blueprint (google.cloud.telcoautomation_v1.types.Blueprint):
            Required. The ``Blueprint`` to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    blueprint_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    blueprint: "Blueprint" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="Blueprint",
    )


class UpdateBlueprintRequest(proto.Message):
    r"""Request object for ``UpdateBlueprint``.

    Attributes:
        blueprint (google.cloud.telcoautomation_v1.types.Blueprint):
            Required. The ``blueprint`` to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. Update mask is used to specify the fields to be
            overwritten in the ``blueprint`` resource by the update.
    """

    blueprint: "Blueprint" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="Blueprint",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class GetBlueprintRequest(proto.Message):
    r"""Request object for ``GetBlueprint``.

    Attributes:
        name (str):
            Required. The name of the blueprint. Case 1: If the name
            provided in the request is {blueprint_id}@{revision_id},
            then the revision with revision_id will be returned. Case 2:
            If the name provided in the request is {blueprint}, then the
            current state of the blueprint is returned.
        view (google.cloud.telcoautomation_v1.types.BlueprintView):
            Optional. Defines the type of view of the blueprint. When
            field is not present BLUEPRINT_VIEW_BASIC is considered as
            default.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    view: "BlueprintView" = proto.Field(
        proto.ENUM,
        number=2,
        enum="BlueprintView",
    )


class DeleteBlueprintRequest(proto.Message):
    r"""Request object for ``DeleteBlueprint``.

    Attributes:
        name (str):
            Required. The name of blueprint to delete. Blueprint name
            should be in the format {blueprint_id}, if
            {blueprint_id}@{revision_id} is passed then the API throws
            invalid argument.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListBlueprintsRequest(proto.Message):
    r"""Request object for ``ListBlueprints``.

    Attributes:
        parent (str):
            Required. The name of parent orchestration cluster resource.
            Format should be -
            "projects/{project_id}/locations/{location_name}/orchestrationClusters/{orchestration_cluster}".
        filter (str):
            Optional. Filtering only supports equality on blueprint
            state. It should be in the form: "state = DRAFT". ``OR``
            operator can be used to get response for multiple states.
            e.g. "state = DRAFT OR state = PROPOSED".
        page_size (int):
            Optional. The maximum number of blueprints to
            return per page.
        page_token (str):
            Optional. The page token, received from a
            previous ListBlueprints call. It can be provided
            to retrieve the subsequent page.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListBlueprintsResponse(proto.Message):
    r"""Response object for ``ListBlueprints``.

    Attributes:
        blueprints (MutableSequence[google.cloud.telcoautomation_v1.types.Blueprint]):
            The list of requested blueprints.
        next_page_token (str):
            A token that can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    blueprints: MutableSequence["Blueprint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Blueprint",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ApproveBlueprintRequest(proto.Message):
    r"""Request object for ``ApproveBlueprint``.

    Attributes:
        name (str):
            Required. The name of the blueprint to
            approve. The blueprint must be in Proposed
            state. A new revision is committed on approval.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ProposeBlueprintRequest(proto.Message):
    r"""Request object for ``ProposeBlueprint``.

    Attributes:
        name (str):
            Required. The name of the blueprint being
            proposed.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RejectBlueprintRequest(proto.Message):
    r"""Request object for ``RejectBlueprint``.

    Attributes:
        name (str):
            Required. The name of the blueprint being
            rejected.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListBlueprintRevisionsRequest(proto.Message):
    r"""Request object for ``ListBlueprintRevisions``.

    Attributes:
        name (str):
            Required. The name of the blueprint to list
            revisions for.
        page_size (int):
            The maximum number of revisions to return per
            page.
        page_token (str):
            The page token, received from a previous
            ListBlueprintRevisions call It can be provided
            to retrieve the subsequent page.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListBlueprintRevisionsResponse(proto.Message):
    r"""Response object for ``ListBlueprintRevisions``.

    Attributes:
        blueprints (MutableSequence[google.cloud.telcoautomation_v1.types.Blueprint]):
            The revisions of the blueprint.
        next_page_token (str):
            A token that can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    blueprints: MutableSequence["Blueprint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Blueprint",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SearchBlueprintRevisionsRequest(proto.Message):
    r"""Request object for ``SearchBlueprintRevisions``.

    Attributes:
        parent (str):
            Required. The name of parent orchestration cluster resource.
            Format should be -
            "projects/{project_id}/locations/{location_name}/orchestrationClusters/{orchestration_cluster}".
        query (str):
            Required. Supported queries:

            1. ""                       : Lists all
                revisions across all blueprints.
            2. "latest=true"            : Lists latest
                revisions across all blueprints.
            3. "name={name}"            : Lists all
                revisions of blueprint with name {name}.
            4. "name={name} latest=true": Lists latest
                revision of blueprint with name {name}
        page_size (int):
            Optional. The maximum number of blueprints
            revisions to return per page. max page size =
            100, default page size = 20.
        page_token (str):
            Optional. The page token, received from a
            previous search call. It can be provided to
            retrieve the subsequent page.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    query: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )


class SearchBlueprintRevisionsResponse(proto.Message):
    r"""Response object for ``SearchBlueprintRevisions``.

    Attributes:
        blueprints (MutableSequence[google.cloud.telcoautomation_v1.types.Blueprint]):
            The list of requested blueprint revisions.
        next_page_token (str):
            A token that can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    blueprints: MutableSequence["Blueprint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Blueprint",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DiscardBlueprintChangesRequest(proto.Message):
    r"""Request object for ``DiscardBlueprintChanges``.

    Attributes:
        name (str):
            Required. The name of the blueprint of which
            changes are being discarded.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class DiscardBlueprintChangesResponse(proto.Message):
    r"""Response object for ``DiscardBlueprintChanges``."""


class ListPublicBlueprintsRequest(proto.Message):
    r"""Request object for ``ListPublicBlueprints``.

    Attributes:
        parent (str):
            Required. Parent value of public blueprint. Format should be
            - "projects/{project_id}/locations/{location_name}".
        page_size (int):
            Optional. Requested page size. Server may
            return fewer items than requested. If
            unspecified, server will pick an appropriate
            default.
        page_token (str):
            Optional. A token identifying a page of
            results the server should return.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListPublicBlueprintsResponse(proto.Message):
    r"""Response object for ``ListPublicBlueprints``.

    Attributes:
        public_blueprints (MutableSequence[google.cloud.telcoautomation_v1.types.PublicBlueprint]):
            The list of public blueprints to return.
        next_page_token (str):
            Output only. A token identifying a page of
            results the server should return.
    """

    @property
    def raw_page(self):
        return self

    public_blueprints: MutableSequence["PublicBlueprint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="PublicBlueprint",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetPublicBlueprintRequest(proto.Message):
    r"""Request object for ``GetPublicBlueprint``.

    Attributes:
        name (str):
            Required. The name of the public blueprint.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateDeploymentRequest(proto.Message):
    r"""Request object for ``CreateDeployment``.

    Attributes:
        parent (str):
            Required. The name of parent resource. Format should be -
            "projects/{project_id}/locations/{location_name}/orchestrationClusters/{orchestration_cluster}".
        deployment_id (str):
            Optional. The name of the deployment.
        deployment (google.cloud.telcoautomation_v1.types.Deployment):
            Required. The ``Deployment`` to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    deployment_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    deployment: "Deployment" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="Deployment",
    )


class UpdateDeploymentRequest(proto.Message):
    r"""Request object for ``UpdateDeployment``.

    Attributes:
        deployment (google.cloud.telcoautomation_v1.types.Deployment):
            Required. The ``deployment`` to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. Update mask is used to specify the fields to be
            overwritten in the ``deployment`` resource by the update.
    """

    deployment: "Deployment" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="Deployment",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class GetDeploymentRequest(proto.Message):
    r"""Request object for ``GetDeployment``.

    Attributes:
        name (str):
            Required. The name of the deployment. Case 1: If the name
            provided in the request is {deployment_id}@{revision_id},
            then the revision with revision_id will be returned. Case 2:
            If the name provided in the request is {deployment}, then
            the current state of the deployment is returned.
        view (google.cloud.telcoautomation_v1.types.DeploymentView):
            Optional. Defines the type of view of the deployment. When
            field is not present VIEW_BASIC is considered as default.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    view: "DeploymentView" = proto.Field(
        proto.ENUM,
        number=2,
        enum="DeploymentView",
    )


class RemoveDeploymentRequest(proto.Message):
    r"""Request object for ``RemoveDeployment``.

    Attributes:
        name (str):
            Required. The name of deployment to initiate
            delete.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListDeploymentsRequest(proto.Message):
    r"""Request object for ``ListDeployments``.

    Attributes:
        parent (str):
            Required. The name of parent orchestration cluster resource.
            Format should be -
            "projects/{project_id}/locations/{location_name}/orchestrationClusters/{orchestration_cluster}".
        filter (str):
            Optional. Filtering only supports equality on deployment
            state. It should be in the form: "state = DRAFT". ``OR``
            operator can be used to get response for multiple states.
            e.g. "state = DRAFT OR state = APPLIED".
        page_size (int):
            Optional. The maximum number of deployments
            to return per page.
        page_token (str):
            Optional. The page token, received from a
            previous ListDeployments call. It can be
            provided to retrieve the subsequent page.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListDeploymentsResponse(proto.Message):
    r"""Response object for ``ListDeployments``.

    Attributes:
        deployments (MutableSequence[google.cloud.telcoautomation_v1.types.Deployment]):
            The list of requested deployments.
        next_page_token (str):
            A token that can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    deployments: MutableSequence["Deployment"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Deployment",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ListDeploymentRevisionsRequest(proto.Message):
    r"""Request for listing all revisions of a deployment.

    Attributes:
        name (str):
            Required. The name of the deployment to list
            revisions for.
        page_size (int):
            Optional. The maximum number of revisions to
            return per page.
        page_token (str):
            Optional. The page token, received from a
            previous ListDeploymentRevisions call Provide
            this to retrieve the subsequent page.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListDeploymentRevisionsResponse(proto.Message):
    r"""List of deployment revisions for a given deployment.

    Attributes:
        deployments (MutableSequence[google.cloud.telcoautomation_v1.types.Deployment]):
            The revisions of the deployment.
        next_page_token (str):
            A token that can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    deployments: MutableSequence["Deployment"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Deployment",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SearchDeploymentRevisionsRequest(proto.Message):
    r"""Request object for ``SearchDeploymentRevisions``.

    Attributes:
        parent (str):
            Required. The name of parent orchestration cluster resource.
            Format should be -
            "projects/{project_id}/locations/{location_name}/orchestrationClusters/{orchestration_cluster}".
        query (str):
            Required. Supported queries:

            1. ""                       : Lists all
                revisions across all deployments.
            2. "latest=true"            : Lists latest
                revisions across all deployments.
            3. "name={name}"            : Lists all
                revisions of deployment with name {name}.
            4. "name={name} latest=true": Lists latest
                revision of deployment with name {name}
        page_size (int):
            Optional. The maximum number of deployment
            revisions to return per page. max page size =
            100, default page size = 20.
        page_token (str):
            Optional. The page token, received from a
            previous search call. It can be provided to
            retrieve the subsequent page.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    query: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )


class SearchDeploymentRevisionsResponse(proto.Message):
    r"""Response object for ``SearchDeploymentRevisions``.

    Attributes:
        deployments (MutableSequence[google.cloud.telcoautomation_v1.types.Deployment]):
            The list of requested deployment revisions.
        next_page_token (str):
            A token that can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    deployments: MutableSequence["Deployment"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Deployment",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DiscardDeploymentChangesRequest(proto.Message):
    r"""Request object for ``DiscardDeploymentChanges``.

    Attributes:
        name (str):
            Required. The name of the deployment of which
            changes are being discarded.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class DiscardDeploymentChangesResponse(proto.Message):
    r"""Response object for ``DiscardDeploymentChanges``."""


class ApplyDeploymentRequest(proto.Message):
    r"""Request object for ``ApplyDeployment``. The resources in given
    deployment gets applied to Orchestration Cluster. A new revision is
    created when a deployment is applied.

    Attributes:
        name (str):
            Required. The name of the deployment to apply
            to orchestration cluster.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ComputeDeploymentStatusRequest(proto.Message):
    r"""Request object for ``ComputeDeploymentStatus``.

    Attributes:
        name (str):
            Required. The name of the deployment without
            revisionID.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ComputeDeploymentStatusResponse(proto.Message):
    r"""Response object for ``ComputeDeploymentStatus``.

    Attributes:
        name (str):
            The name of the deployment.
        aggregated_status (google.cloud.telcoautomation_v1.types.Status):
            Output only. Aggregated status of a
            deployment.
        resource_statuses (MutableSequence[google.cloud.telcoautomation_v1.types.ResourceStatus]):
            Output only. Resource level status details in
            deployments.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    aggregated_status: "Status" = proto.Field(
        proto.ENUM,
        number=2,
        enum="Status",
    )
    resource_statuses: MutableSequence["ResourceStatus"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="ResourceStatus",
    )


class RollbackDeploymentRequest(proto.Message):
    r"""Request object for ``RollbackDeployment``.

    Attributes:
        name (str):
            Required. Name of the deployment.
        revision_id (str):
            Required. The revision id of deployment to
            roll back to.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    revision_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


class OperationMetadata(proto.Message):
    r"""Represents the metadata of the long-running operation.

    Attributes:
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time the operation was
            created.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time the operation finished
            running.
        target (str):
            Output only. Server-defined resource path for
            the target of the operation.
        verb (str):
            Output only. Name of the verb executed by the
            operation.
        status_message (str):
            Output only. Human-readable status of the
            operation, if any.
        requested_cancellation (bool):
            Output only. Identifies whether the user has requested
            cancellation of the operation. Operations that have been
            cancelled successfully have [Operation.error][] value with a
            [google.rpc.Status.code][google.rpc.Status.code] of 1,
            corresponding to ``Code.CANCELLED``.
        api_version (str):
            Output only. API version used to start the
            operation.
    """

    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    target: str = proto.Field(
        proto.STRING,
        number=3,
    )
    verb: str = proto.Field(
        proto.STRING,
        number=4,
    )
    status_message: str = proto.Field(
        proto.STRING,
        number=5,
    )
    requested_cancellation: bool = proto.Field(
        proto.BOOL,
        number=6,
    )
    api_version: str = proto.Field(
        proto.STRING,
        number=7,
    )


class GetHydratedDeploymentRequest(proto.Message):
    r"""Request object for ``GetHydratedDeployment``.

    Attributes:
        name (str):
            Required. Name of the hydrated deployment.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListHydratedDeploymentsRequest(proto.Message):
    r"""Request object for ``ListHydratedDeployments``.

    Attributes:
        parent (str):
            Required. The deployment managing the
            hydrated deployments.
        page_size (int):
            Optional. The maximum number of hydrated
            deployments to return. The service may return
            fewer than this value. If unspecified, at most
            50 hydrated deployments will be returned. The
            maximum value is 1000. Values above 1000 will be
            set to 1000.
        page_token (str):
            Optional. The page token, received from a
            previous ListHydratedDeployments call. Provide
            this to retrieve the subsequent page.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListHydratedDeploymentsResponse(proto.Message):
    r"""Response object for ``ListHydratedDeployments``.

    Attributes:
        hydrated_deployments (MutableSequence[google.cloud.telcoautomation_v1.types.HydratedDeployment]):
            The list of hydrated deployments.
        next_page_token (str):
            A token that can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    hydrated_deployments: MutableSequence["HydratedDeployment"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="HydratedDeployment",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class UpdateHydratedDeploymentRequest(proto.Message):
    r"""Request object for ``UpdateHydratedDeployment``.

    Attributes:
        hydrated_deployment (google.cloud.telcoautomation_v1.types.HydratedDeployment):
            Required. The hydrated deployment to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The list of fields to update. Update mask supports
            a special value ``*`` which fully replaces (equivalent to
            PUT) the resource provided.
    """

    hydrated_deployment: "HydratedDeployment" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="HydratedDeployment",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class ApplyHydratedDeploymentRequest(proto.Message):
    r"""Request for applying a hydrated deployment.

    Attributes:
        name (str):
            Required. The name of the hydrated deployment
            to apply.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ManagementConfig(proto.Message):
    r"""Configuration of the cluster management

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        standard_management_config (google.cloud.telcoautomation_v1.types.StandardManagementConfig):
            Configuration of the standard (GKE) cluster
            management

            This field is a member of `oneof`_ ``oneof_config``.
        full_management_config (google.cloud.telcoautomation_v1.types.FullManagementConfig):
            Configuration of the full (Autopilot) cluster
            management. Full cluster management is a preview
            feature.

            This field is a member of `oneof`_ ``oneof_config``.
    """

    standard_management_config: "StandardManagementConfig" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="oneof_config",
        message="StandardManagementConfig",
    )
    full_management_config: "FullManagementConfig" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="oneof_config",
        message="FullManagementConfig",
    )


class StandardManagementConfig(proto.Message):
    r"""Configuration of the standard (GKE) cluster management.

    Attributes:
        network (str):
            Optional. Name of the VPC Network to put the
            GKE cluster and nodes in. The VPC will be
            created if it doesn't exist.
        subnet (str):
            Optional. Specifies the subnet that the
            interface will be part of. Network key must be
            specified and the subnet must be a subnetwork of
            the specified network.
        master_ipv4_cidr_block (str):
            Optional. The /28 network that the masters
            will use. It should be free within the network.
        cluster_cidr_block (str):
            Optional. The IP address range for the
            cluster pod IPs. Set to blank to have a range
            chosen with the default size. Set to /netmask
            (e.g. /14) to have a range chosen with a
            specific netmask. Set to a CIDR notation (e.g.
            10.96.0.0/14) from the RFC-1918 private networks
            (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16)
            to pick a specific range to use.
        services_cidr_block (str):
            Optional. The IP address range for the
            cluster service IPs. Set to blank to have a
            range chosen with the default size. Set to
            /netmask (e.g. /14) to have a range chosen with
            a specific netmask. Set to a CIDR notation (e.g.
            10.96.0.0/14) from the RFC-1918 private networks
            (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16)
            to pick a specific range to use.
        cluster_named_range (str):
            Optional. The name of the existing secondary range in the
            cluster's subnetwork to use for pod IP addresses.
            Alternatively, cluster_cidr_block can be used to
            automatically create a GKE-managed one.
        services_named_range (str):
            Optional. The name of the existing secondary range in the
            cluster's subnetwork to use for service ClusterIPs.
            Alternatively, services_cidr_block can be used to
            automatically create a GKE-managed one.
        master_authorized_networks_config (google.cloud.telcoautomation_v1.types.MasterAuthorizedNetworksConfig):
            Optional. Master Authorized Network that supports multiple
            CIDR blocks. Allows access to the k8s master from multiple
            blocks. It cannot be set at the same time with the field
            man_block.
    """

    network: str = proto.Field(
        proto.STRING,
        number=1,
    )
    subnet: str = proto.Field(
        proto.STRING,
        number=2,
    )
    master_ipv4_cidr_block: str = proto.Field(
        proto.STRING,
        number=3,
    )
    cluster_cidr_block: str = proto.Field(
        proto.STRING,
        number=4,
    )
    services_cidr_block: str = proto.Field(
        proto.STRING,
        number=5,
    )
    cluster_named_range: str = proto.Field(
        proto.STRING,
        number=6,
    )
    services_named_range: str = proto.Field(
        proto.STRING,
        number=7,
    )
    master_authorized_networks_config: "MasterAuthorizedNetworksConfig" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="MasterAuthorizedNetworksConfig",
    )


class FullManagementConfig(proto.Message):
    r"""Configuration of the full (Autopilot) cluster management

    Attributes:
        network (str):
            Optional. Name of the VPC Network to put the
            GKE cluster and nodes in. The VPC will be
            created if it doesn't exist.
        subnet (str):
            Optional. Specifies the subnet that the
            interface will be part of. Network key must be
            specified and the subnet must be a subnetwork of
            the specified network.
        master_ipv4_cidr_block (str):
            Optional. The /28 network that the masters
            will use.
        cluster_cidr_block (str):
            Optional. The IP address range for the
            cluster pod IPs. Set to blank to have a range
            chosen with the default size. Set to /netmask
            (e.g. /14) to have a range chosen with a
            specific netmask. Set to a CIDR notation (e.g.
            10.96.0.0/14) from the RFC-1918 private networks
            (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16)
            to pick a specific range to use.
        services_cidr_block (str):
            Optional. The IP address range for the
            cluster service IPs. Set to blank to have a
            range chosen with the default size. Set to
            /netmask (e.g. /14) to have a range chosen with
            a specific netmask. Set to a CIDR notation (e.g.
            10.96.0.0/14) from the RFC-1918 private networks
            (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16)
            to pick a specific range to use.
        cluster_named_range (str):
            Optional. The name of the existing secondary range in the
            cluster's subnetwork to use for pod IP addresses.
            Alternatively, cluster_cidr_block can be used to
            automatically create a GKE-managed one.
        services_named_range (str):
            Optional. The name of the existing secondary range in the
            cluster's subnetwork to use for service ClusterIPs.
            Alternatively, services_cidr_block can be used to
            automatically create a GKE-managed one.
        master_authorized_networks_config (google.cloud.telcoautomation_v1.types.MasterAuthorizedNetworksConfig):
            Optional. Master Authorized Network that supports multiple
            CIDR blocks. Allows access to the k8s master from multiple
            blocks. It cannot be set at the same time with the field
            man_block.
    """

    network: str = proto.Field(
        proto.STRING,
        number=1,
    )
    subnet: str = proto.Field(
        proto.STRING,
        number=2,
    )
    master_ipv4_cidr_block: str = proto.Field(
        proto.STRING,
        number=3,
    )
    cluster_cidr_block: str = proto.Field(
        proto.STRING,
        number=4,
    )
    services_cidr_block: str = proto.Field(
        proto.STRING,
        number=5,
    )
    cluster_named_range: str = proto.Field(
        proto.STRING,
        number=6,
    )
    services_named_range: str = proto.Field(
        proto.STRING,
        number=7,
    )
    master_authorized_networks_config: "MasterAuthorizedNetworksConfig" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="MasterAuthorizedNetworksConfig",
    )


class MasterAuthorizedNetworksConfig(proto.Message):
    r"""Configuration of the Master Authorized Network that support
    multiple CIDRs

    Attributes:
        cidr_blocks (MutableSequence[google.cloud.telcoautomation_v1.types.MasterAuthorizedNetworksConfig.CidrBlock]):
            Optional. cidr_blocks define up to 50 external networks that
            could access Kubernetes master through HTTPS.
    """

    class CidrBlock(proto.Message):
        r"""CidrBlock contains an optional name and one CIDR block.

        Attributes:
            display_name (str):
                Optional. display_name is an optional field for users to
                identify CIDR blocks.
            cidr_block (str):
                Optional. cidr_block must be specified in CIDR notation when
                using master_authorized_networks_config. Currently, the user
                could still use the deprecated man_block field, so this
                field is currently optional, but will be required in the
                future.
        """

        display_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        cidr_block: str = proto.Field(
            proto.STRING,
            number=2,
        )

    cidr_blocks: MutableSequence[CidrBlock] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=CidrBlock,
    )


class File(proto.Message):
    r"""File represents a yaml file present in a blueprint's package.

    Attributes:
        path (str):
            Required. Path of the file in package. e.g.
            ``gdce/v1/cluster.yaml``
        content (str):
            Optional. The contents of a file in string
            format.
        deleted (bool):
            Optional. Signifies whether a file is marked
            for deletion.
        editable (bool):
            Optional. Indicates whether changes are
            allowed to a file. If the field is not set, the
            file cannot be edited.
    """

    path: str = proto.Field(
        proto.STRING,
        number=1,
    )
    content: str = proto.Field(
        proto.STRING,
        number=2,
    )
    deleted: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    editable: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class ResourceStatus(proto.Message):
    r"""Status of a deployment resource.

    Attributes:
        name (str):
            Name of the resource.
        resource_namespace (str):
            Namespace of the resource.
        group (str):
            Group to which the resource belongs to.
        version (str):
            Version of the resource.
        kind (str):
            Kind of the resource.
        resource_type (google.cloud.telcoautomation_v1.types.ResourceType):
            Output only. Resource type.
        status (google.cloud.telcoautomation_v1.types.Status):
            Output only. Status of the resource.
        nf_deploy_status (google.cloud.telcoautomation_v1.types.NFDeployStatus):
            Output only. Detailed status of NFDeploy.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    resource_namespace: str = proto.Field(
        proto.STRING,
        number=2,
    )
    group: str = proto.Field(
        proto.STRING,
        number=3,
    )
    version: str = proto.Field(
        proto.STRING,
        number=4,
    )
    kind: str = proto.Field(
        proto.STRING,
        number=5,
    )
    resource_type: "ResourceType" = proto.Field(
        proto.ENUM,
        number=6,
        enum="ResourceType",
    )
    status: "Status" = proto.Field(
        proto.ENUM,
        number=7,
        enum="Status",
    )
    nf_deploy_status: "NFDeployStatus" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="NFDeployStatus",
    )


class NFDeployStatus(proto.Message):
    r"""Deployment status of NFDeploy.

    Attributes:
        targeted_nfs (int):
            Output only. Total number of NFs targeted by
            this deployment
        ready_nfs (int):
            Output only. Total number of NFs targeted by
            this deployment with a Ready Condition set.
        sites (MutableSequence[google.cloud.telcoautomation_v1.types.NFDeploySiteStatus]):
            Output only. Per-Site Status.
    """

    targeted_nfs: int = proto.Field(
        proto.INT32,
        number=1,
    )
    ready_nfs: int = proto.Field(
        proto.INT32,
        number=2,
    )
    sites: MutableSequence["NFDeploySiteStatus"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="NFDeploySiteStatus",
    )


class NFDeploySiteStatus(proto.Message):
    r"""Per-Site Status.

    Attributes:
        site (str):
            Output only. Site id.
        pending_deletion (bool):
            Output only. If true, the Site Deletion is in
            progress.
        hydration (google.cloud.telcoautomation_v1.types.HydrationStatus):
            Output only. Hydration status.
        workload (google.cloud.telcoautomation_v1.types.WorkloadStatus):
            Output only. Workload status.
    """

    site: str = proto.Field(
        proto.STRING,
        number=1,
    )
    pending_deletion: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    hydration: "HydrationStatus" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="HydrationStatus",
    )
    workload: "WorkloadStatus" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="WorkloadStatus",
    )


class HydrationStatus(proto.Message):
    r"""Hydration status.

    Attributes:
        site_version (google.cloud.telcoautomation_v1.types.SiteVersion):
            Output only. SiteVersion Hydration is
            targeting.
        status (str):
            Output only. Status.
    """

    site_version: "SiteVersion" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="SiteVersion",
    )
    status: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SiteVersion(proto.Message):
    r"""SiteVersion Hydration is targeting.

    Attributes:
        nf_vendor (str):
            Output only. NF vendor.
        nf_type (str):
            Output only. NF vendor type.
        nf_version (str):
            Output only. NF version.
    """

    nf_vendor: str = proto.Field(
        proto.STRING,
        number=1,
    )
    nf_type: str = proto.Field(
        proto.STRING,
        number=2,
    )
    nf_version: str = proto.Field(
        proto.STRING,
        number=3,
    )


class WorkloadStatus(proto.Message):
    r"""Workload status.

    Attributes:
        site_version (google.cloud.telcoautomation_v1.types.SiteVersion):
            Output only. SiteVersion running in the
            workload cluster.
        status (str):
            Output only. Status.
    """

    site_version: "SiteVersion" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="SiteVersion",
    )
    status: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import inspect
import json
import logging as std_logging
import pickle
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1, grpc_helpers_async, operations_v1
from google.api_core import retry_async as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.cloud.location import locations_pb2  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf.json_format import MessageToJson
import google.protobuf.message
import grpc  # type: ignore
from grpc.experimental import aio  # type: ignore
import proto  # type: ignore

from google.cloud.telcoautomation_v1.types import telcoautomation

from .base import DEFAULT_CLIENT_INFO, TelcoAutomationTransport
from .grpc import TelcoAutomationGrpcTransport

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class _LoggingClientAIOInterceptor(
    grpc.aio.UnaryUnaryClientInterceptor
):  # pragma: NO COVER
    async def intercept_unary_unary(self, continuation, client_call_details, request):
        logging_enabled = CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        )
        if logging_enabled:  # pragma: NO COVER
            request_metadata = client_call_details.metadata
            if isinstance(request, proto.Message):
                request_payload = type(request).to_json(request)
            elif isinstance(request, google.protobuf.message.Message):
                request_payload = MessageToJson(request)
            else:
                request_payload = f"{type(request).__name__}: {pickle.dumps(request)}"

            request_metadata = {
                key: value.decode("utf-8") if isinstance(value, bytes) else value
                for key, value in request_metadata
            }
            grpc_request = {
                "payload": request_payload,
                "requestMethod": "grpc",
                "metadata": dict(request_metadata),
            }
            _LOGGER.debug(
                f"Sending request for {client_call_details.method}",
                extra={
                    "serviceName": "google.cloud.telcoautomation.v1.TelcoAutomation",
                    "rpcName": str(client_call_details.method),
                    "request": grpc_request,
                    "metadata": grpc_request["metadata"],
                },
            )
        response = await continuation(client_call_details, request)
        if logging_enabled:  # pragma: NO COVER
            response_metadata = await response.trailing_metadata()
            # Convert gRPC metadata `<class 'grpc.aio._metadata.Metadata'>` to list of tuples
            metadata = (
                dict([(k, str(v)) for k, v in response_metadata])
                if response_metadata
                else None
            )
            result = await response
            if isinstance(result, proto.Message):
                response_payload = type(result).to_json(result)
            elif isinstance(result, google.protobuf.message.Message):
                response_payload = MessageToJson(result)
            else:
                response_payload = f"{type(result).__name__}: {pickle.dumps(result)}"
            grpc_response = {
                "payload": response_payload,
                "metadata": metadata,
                "status": "OK",
            }
            _LOGGER.debug(
                f"Received response to rpc {client_call_details.method}.",
                extra={
                    "serviceName": "google.cloud.telcoautomation.v1.TelcoAutomation",
                    "rpcName": str(client_call_details.method),
                    "response": grpc_response,
                    "metadata": grpc_response["metadata"],
                },
            )
        return response


class TelcoAutomationGrpcAsyncIOTransport(TelcoAutomationTransport):
    """gRPC AsyncIO backend transport for TelcoAutomation.

    TelcoAutomation Service manages the control plane cluster
    a.k.a. Orchestration Cluster (GKE cluster with config
    controller) of TNA. It also exposes blueprint APIs which manages
    the lifecycle of blueprints that control the infrastructure
    setup (e.g GDCE clusters) and deployment of network functions.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "telcoautomation.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): Deprecated. A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`. This argument will be
                removed in the next major version of this library.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "telcoautomation.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: Optional[Union[aio.Channel, Callable[..., aio.Channel]]] = None,
        api_mtls_endpoint: Optional[str] = None,
        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'telcoautomation.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if a ``channel`` instance is provided.
            credentials_file (Optional[str]): Deprecated. A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if a ``channel`` instance is provided.
                This argument will be removed in the next major version of this library.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[Union[aio.Channel, Callable[..., aio.Channel]]]):
                A ``Channel`` instance through which to make calls, or a Callable
                that constructs and returns one. If set to None, ``self.create_channel``
                is used to create the channel. If a Callable is given, it will be called
                with the same arguments as used in ``self.create_channel``.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if a ``channel`` instance is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if a ``channel`` instance or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}
        self._operations_client: Optional[operations_v1.OperationsAsyncClient] = None

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if isinstance(channel, aio.Channel):
            # Ignore credentials if a channel was passed.
            credentials = None
            self._ignore_credentials = True
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            # initialize with the provided callable or the default channel
            channel_init = channel or type(self).create_channel
            self._grpc_channel = channel_init(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        self._interceptor = _LoggingClientAIOInterceptor()
        self._grpc_channel._unary_unary_interceptors.append(self._interceptor)
        self._logged_channel = self._grpc_channel
        self._wrap_with_kind = (
            "kind" in inspect.signature(gapic_v1.method_async.wrap_method).parameters
        )
        # Wrap messages. This must be done after self._logged_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def operations_client(self) -> operations_v1.OperationsAsyncClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Quick check: Only create a new client if we do not already have one.
        if self._operations_client is None:
            self._operations_client = operations_v1.OperationsAsyncClient(
                self._logged_channel
            )

        # Return the client from cache.
        return self._operations_client

    @property
    def list_orchestration_clusters(
        self,
    ) -> Callable[
        [telcoautomation.ListOrchestrationClustersRequest],
        Awaitable[telcoautomation.ListOrchestrationClustersResponse],
    ]:
        r"""Return a callable for the list orchestration clusters method over gRPC.

        Lists OrchestrationClusters in a given project and
        location.

        Returns:
            Callable[[~.ListOrchestrationClustersRequest],
                    Awaitable[~.ListOrchestrationClustersResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_orchestration_clusters" not in self._stubs:
            self._stubs[
                "list_orchestration_clusters"
            ] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListOrchestrationClusters",
                request_serializer=telcoautomation.ListOrchestrationClustersRequest.serialize,
                response_deserializer=telcoautomation.ListOrchestrationClustersResponse.deserialize,
            )
        return self._stubs["list_orchestration_clusters"]

    @property
    def get_orchestration_cluster(
        self,
    ) -> Callable[
        [telcoautomation.GetOrchestrationClusterRequest],
        Awaitable[telcoautomation.OrchestrationCluster],
    ]:
        r"""Return a callable for the get orchestration cluster method over gRPC.

        Gets details of a single OrchestrationCluster.

        Returns:
            Callable[[~.GetOrchestrationClusterRequest],
                    Awaitable[~.OrchestrationCluster]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_orchestration_cluster" not in self._stubs:
            self._stubs["get_orchestration_cluster"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/GetOrchestrationCluster",
                request_serializer=telcoautomation.GetOrchestrationClusterRequest.serialize,
                response_deserializer=telcoautomation.OrchestrationCluster.deserialize,
            )
        return self._stubs["get_orchestration_cluster"]

    @property
    def create_orchestration_cluster(
        self,
    ) -> Callable[
        [telcoautomation.CreateOrchestrationClusterRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the create orchestration cluster method over gRPC.

        Creates a new OrchestrationCluster in a given project
        and location.

        Returns:
            Callable[[~.CreateOrchestrationClusterRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_orchestration_cluster" not in self._stubs:
            self._stubs[
                "create_orchestration_cluster"
            ] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/CreateOrchestrationCluster",
                request_serializer=telcoautomation.CreateOrchestrationClusterRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_orchestration_cluster"]

    @property
    def delete_orchestration_cluster(
        self,
    ) -> Callable[
        [telcoautomation.DeleteOrchestrationClusterRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the delete orchestration cluster method over gRPC.

        Deletes a single OrchestrationCluster.

        Returns:
            Callable[[~.DeleteOrchestrationClusterRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_orchestration_cluster" not in self._stubs:
            self._stubs[
                "delete_orchestration_cluster"
            ] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/DeleteOrchestrationCluster",
                request_serializer=telcoautomation.DeleteOrchestrationClusterRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["delete_orchestration_cluster"]

    @property
    def list_edge_slms(
        self,
    ) -> Callable[
        [telcoautomation.ListEdgeSlmsRequest],
        Awaitable[telcoautomation.ListEdgeSlmsResponse],
    ]:
        r"""Return a callable for the list edge slms method over gRPC.

        Lists EdgeSlms in a given project and location.

        Returns:
            Callable[[~.ListEdgeSlmsRequest],
                    Awaitable[~.ListEdgeSlmsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_edge_slms" not in self._stubs:
            self._stubs["list_edge_slms"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListEdgeSlms",
                request_serializer=telcoautomation.ListEdgeSlmsRequest.serialize,
                response_deserializer=telcoautomation.ListEdgeSlmsResponse.deserialize,
            )
        return self._stubs["list_edge_slms"]

    @property
    def get_edge_slm(
        self,
    ) -> Callable[
        [telcoautomation.GetEdgeSlmRequest], Awaitable[telcoautomation.EdgeSlm]
    ]:
        r"""Return a callable for the get edge slm method over gRPC.

        Gets details of a single EdgeSlm.

        Returns:
            Callable[[~.GetEdgeSlmRequest],
                    Awaitable[~.EdgeSlm]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_edge_slm" not in self._stubs:
            self._stubs["get_edge_slm"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/GetEdgeSlm",
                request_serializer=telcoautomation.GetEdgeSlmRequest.serialize,
                response_deserializer=telcoautomation.EdgeSlm.deserialize,
            )
        return self._stubs["get_edge_slm"]

    @property
    def create_edge_slm(
        self,
    ) -> Callable[
        [telcoautomation.CreateEdgeSlmRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the create edge slm method over gRPC.

        Creates a new EdgeSlm in a given project and
        location.

        Returns:
            Callable[[~.CreateEdgeSlmRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_edge_slm" not in self._stubs:
            self._stubs["create_edge_slm"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/CreateEdgeSlm",
                request_serializer=telcoautomation.CreateEdgeSlmRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_edge_slm"]

    @property
    def delete_edge_slm(
        self,
    ) -> Callable[
        [telcoautomation.DeleteEdgeSlmRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the delete edge slm method over gRPC.

        Deletes a single EdgeSlm.

        Returns:
            Callable[[~.DeleteEdgeSlmRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_edge_slm" not in self._stubs:
            self._stubs["delete_edge_slm"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/DeleteEdgeSlm",
                request_serializer=telcoautomation.DeleteEdgeSlmRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["delete_edge_slm"]

    @property
    def create_blueprint(
        self,
    ) -> Callable[
        [telcoautomation.CreateBlueprintRequest], Awaitable[telcoautomation.Blueprint]
    ]:
        r"""Return a callable for the create blueprint method over gRPC.

        Creates a blueprint.

        Returns:
            Callable[[~.CreateBlueprintRequest],
                    Awaitable[~.Blueprint]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_blueprint" not in self._stubs:
            self._stubs["create_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/CreateBlueprint",
                request_serializer=telcoautomation.CreateBlueprintRequest.serialize,
                response_deserializer=telcoautomation.Blueprint.deserialize,
            )
        return self._stubs["create_blueprint"]

    @property
    def update_blueprint(
        self,
    ) -> Callable[
        [telcoautomation.UpdateBlueprintRequest], Awaitable[telcoautomation.Blueprint]
    ]:
        r"""Return a callable for the update blueprint method over gRPC.

        Updates a blueprint.

        Returns:
            Callable[[~.UpdateBlueprintRequest],
                    Awaitable[~.Blueprint]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_blueprint" not in self._stubs:
            self._stubs["update_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/UpdateBlueprint",
                request_serializer=telcoautomation.UpdateBlueprintRequest.serialize,
                response_deserializer=telcoautomation.Blueprint.deserialize,
            )
        return self._stubs["update_blueprint"]

    @property
    def get_blueprint(
        self,
    ) -> Callable[
        [telcoautomation.GetBlueprintRequest], Awaitable[telcoautomation.Blueprint]
    ]:
        r"""Return a callable for the get blueprint method over gRPC.

        Returns the requested blueprint.

        Returns:
            Callable[[~.GetBlueprintRequest],
                    Awaitable[~.Blueprint]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_blueprint" not in self._stubs:
            self._stubs["get_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/GetBlueprint",
                request_serializer=telcoautomation.GetBlueprintRequest.serialize,
                response_deserializer=telcoautomation.Blueprint.deserialize,
            )
        return self._stubs["get_blueprint"]

    @property
    def delete_blueprint(
        self,
    ) -> Callable[[telcoautomation.DeleteBlueprintRequest], Awaitable[empty_pb2.Empty]]:
        r"""Return a callable for the delete blueprint method over gRPC.

        Deletes a blueprint and all its revisions.

        Returns:
            Callable[[~.DeleteBlueprintRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_blueprint" not in self._stubs:
            self._stubs["delete_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/DeleteBlueprint",
                request_serializer=telcoautomation.DeleteBlueprintRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_blueprint"]

    @property
    def list_blueprints(
        self,
    ) -> Callable[
        [telcoautomation.ListBlueprintsRequest],
        Awaitable[telcoautomation.ListBlueprintsResponse],
    ]:
        r"""Return a callable for the list blueprints method over gRPC.

        List all blueprints.

        Returns:
            Callable[[~.ListBlueprintsRequest],
                    Awaitable[~.ListBlueprintsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_blueprints" not in self._stubs:
            self._stubs["list_blueprints"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListBlueprints",
                request_serializer=telcoautomation.ListBlueprintsRequest.serialize,
                response_deserializer=telcoautomation.ListBlueprintsResponse.deserialize,
            )
        return self._stubs["list_blueprints"]

    @property
    def approve_blueprint(
        self,
    ) -> Callable[
        [telcoautomation.ApproveBlueprintRequest], Awaitable[telcoautomation.Blueprint]
    ]:
        r"""Return a callable for the approve blueprint method over gRPC.

        Approves a blueprint and commits a new revision.

        Returns:
            Callable[[~.ApproveBlueprintRequest],
                    Awaitable[~.Blueprint]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "approve_blueprint" not in self._stubs:
            self._stubs["approve_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ApproveBlueprint",
                request_serializer=telcoautomation.ApproveBlueprintRequest.serialize,
                response_deserializer=telcoautomation.Blueprint.deserialize,
            )
        return self._stubs["approve_blueprint"]

    @property
    def propose_blueprint(
        self,
    ) -> Callable[
        [telcoautomation.ProposeBlueprintRequest], Awaitable[telcoautomation.Blueprint]
    ]:
        r"""Return a callable for the propose blueprint method over gRPC.

        Proposes a blueprint for approval of changes.

        Returns:
            Callable[[~.ProposeBlueprintRequest],
                    Awaitable[~.Blueprint]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "propose_blueprint" not in self._stubs:
            self._stubs["propose_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ProposeBlueprint",
                request_serializer=telcoautomation.ProposeBlueprintRequest.serialize,
                response_deserializer=telcoautomation.Blueprint.deserialize,
            )
        return self._stubs["propose_blueprint"]

    @property
    def reject_blueprint(
        self,
    ) -> Callable[
        [telcoautomation.RejectBlueprintRequest], Awaitable[telcoautomation.Blueprint]
    ]:
        r"""Return a callable for the reject blueprint method over gRPC.

        Rejects a blueprint revision proposal and flips it
        back to Draft state.

        Returns:
            Callable[[~.RejectBlueprintRequest],
                    Awaitable[~.Blueprint]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "reject_blueprint" not in self._stubs:
            self._stubs["reject_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/RejectBlueprint",
                request_serializer=telcoautomation.RejectBlueprintRequest.serialize,
                response_deserializer=telcoautomation.Blueprint.deserialize,
            )
        return self._stubs["reject_blueprint"]

    @property
    def list_blueprint_revisions(
        self,
    ) -> Callable[
        [telcoautomation.ListBlueprintRevisionsRequest],
        Awaitable[telcoautomation.ListBlueprintRevisionsResponse],
    ]:
        r"""Return a callable for the list blueprint revisions method over gRPC.

        List blueprint revisions of a given blueprint.

        Returns:
            Callable[[~.ListBlueprintRevisionsRequest],
                    Awaitable[~.ListBlueprintRevisionsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_blueprint_revisions" not in self._stubs:
            self._stubs["list_blueprint_revisions"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListBlueprintRevisions",
                request_serializer=telcoautomation.ListBlueprintRevisionsRequest.serialize,
                response_deserializer=telcoautomation.ListBlueprintRevisionsResponse.deserialize,
            )
        return self._stubs["list_blueprint_revisions"]

    @property
    def search_blueprint_revisions(
        self,
    ) -> Callable[
        [telcoautomation.SearchBlueprintRevisionsRequest],
        Awaitable[telcoautomation.SearchBlueprintRevisionsResponse],
    ]:
        r"""Return a callable for the search blueprint revisions method over gRPC.

        Searches across blueprint revisions.

        Returns:
            Callable[[~.SearchBlueprintRevisionsRequest],
                    Awaitable[~.SearchBlueprintRevisionsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "search_blueprint_revisions" not in self._stubs:
            self._stubs[
                "search_blueprint_revisions"
            ] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/SearchBlueprintRevisions",
                request_serializer=telcoautomation.SearchBlueprintRevisionsRequest.serialize,
                response_deserializer=telcoautomation.SearchBlueprintRevisionsResponse.deserialize,
            )
        return self._stubs["search_blueprint_revisions"]

    @property
    def search_deployment_revisions(
        self,
    ) -> Callable[
        [telcoautomation.SearchDeploymentRevisionsRequest],
        Awaitable[telcoautomation.SearchDeploymentRevisionsResponse],
    ]:
        r"""Return a callable for the search deployment revisions method over gRPC.

        Searches across deployment revisions.

        Returns:
            Callable[[~.SearchDeploymentRevisionsRequest],
                    Awaitable[~.SearchDeploymentRevisionsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "search_deployment_revisions" not in self._stubs:
            self._stubs[
                "search_deployment_revisions"
            ] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/SearchDeploymentRevisions",
                request_serializer=telcoautomation.SearchDeploymentRevisionsRequest.serialize,
                response_deserializer=telcoautomation.SearchDeploymentRevisionsResponse.deserialize,
            )
        return self._stubs["search_deployment_revisions"]

    @property
    def discard_blueprint_changes(
        self,
    ) -> Callable[
        [telcoautomation.DiscardBlueprintChangesRequest],
        Awaitable[telcoautomation.DiscardBlueprintChangesResponse],
    ]:
        r"""Return a callable for the discard blueprint changes method over gRPC.

        Discards the changes in a blueprint and reverts the
        blueprint to the last approved blueprint revision. No
        changes take place if a blueprint does not have
        revisions.

        Returns:
            Callable[[~.DiscardBlueprintChangesRequest],
                    Awaitable[~.DiscardBlueprintChangesResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "discard_blueprint_changes" not in self._stubs:
            self._stubs["discard_blueprint_changes"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/DiscardBlueprintChanges",
                request_serializer=telcoautomation.DiscardBlueprintChangesRequest.serialize,
                response_deserializer=telcoautomation.DiscardBlueprintChangesResponse.deserialize,
            )
        return self._stubs["discard_blueprint_changes"]

    @property
    def list_public_blueprints(
        self,
    ) -> Callable[
        [telcoautomation.ListPublicBlueprintsRequest],
        Awaitable[telcoautomation.ListPublicBlueprintsResponse],
    ]:
        r"""Return a callable for the list public blueprints method over gRPC.

        Lists the blueprints in TNA's public catalog. Default
        page size = 20, Max Page Size = 100.

        Returns:
            Callable[[~.ListPublicBlueprintsRequest],
                    Awaitable[~.ListPublicBlueprintsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_public_blueprints" not in self._stubs:
            self._stubs["list_public_blueprints"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListPublicBlueprints",
                request_serializer=telcoautomation.ListPublicBlueprintsRequest.serialize,
                response_deserializer=telcoautomation.ListPublicBlueprintsResponse.deserialize,
            )
        return self._stubs["list_public_blueprints"]

    @property
    def get_public_blueprint(
        self,
    ) -> Callable[
        [telcoautomation.GetPublicBlueprintRequest],
        Awaitable[telcoautomation.PublicBlueprint],
    ]:
        r"""Return a callable for the get public blueprint method over gRPC.

        Returns the requested public blueprint.

        Returns:
            Callable[[~.GetPublicBlueprintRequest],
                    Awaitable[~.PublicBlueprint]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_public_blueprint" not in self._stubs:
            self._stubs["get_public_blueprint"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/GetPublicBlueprint",
                request_serializer=telcoautomation.GetPublicBlueprintRequest.serialize,
                response_deserializer=telcoautomation.PublicBlueprint.deserialize,
            )
        return self._stubs["get_public_blueprint"]

    @property
    def create_deployment(
        self,
    ) -> Callable[
        [telcoautomation.CreateDeploymentRequest], Awaitable[telcoautomation.Deployment]
    ]:
        r"""Return a callable for the create deployment method over gRPC.

        Creates a deployment.

        Returns:
            Callable[[~.CreateDeploymentRequest],
                    Awaitable[~.Deployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_deployment" not in self._stubs:
            self._stubs["create_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/CreateDeployment",
                request_serializer=telcoautomation.CreateDeploymentRequest.serialize,
                response_deserializer=telcoautomation.Deployment.deserialize,
            )
        return self._stubs["create_deployment"]

    @property
    def update_deployment(
        self,
    ) -> Callable[
        [telcoautomation.UpdateDeploymentRequest], Awaitable[telcoautomation.Deployment]
    ]:
        r"""Return a callable for the update deployment method over gRPC.

        Updates a deployment.

        Returns:
            Callable[[~.UpdateDeploymentRequest],
                    Awaitable[~.Deployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_deployment" not in self._stubs:
            self._stubs["update_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/UpdateDeployment",
                request_serializer=telcoautomation.UpdateDeploymentRequest.serialize,
                response_deserializer=telcoautomation.Deployment.deserialize,
            )
        return self._stubs["update_deployment"]

    @property
    def get_deployment(
        self,
    ) -> Callable[
        [telcoautomation.GetDeploymentRequest], Awaitable[telcoautomation.Deployment]
    ]:
        r"""Return a callable for the get deployment method over gRPC.

        Returns the requested deployment.

        Returns:
            Callable[[~.GetDeploymentRequest],
                    Awaitable[~.Deployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_deployment" not in self._stubs:
            self._stubs["get_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/GetDeployment",
                request_serializer=telcoautomation.GetDeploymentRequest.serialize,
                response_deserializer=telcoautomation.Deployment.deserialize,
            )
        return self._stubs["get_deployment"]

    @property
    def remove_deployment(
        self,
    ) -> Callable[
        [telcoautomation.RemoveDeploymentRequest], Awaitable[empty_pb2.Empty]
    ]:
        r"""Return a callable for the remove deployment method over gRPC.

        Removes the deployment by marking it as DELETING.
        Post which deployment and it's revisions gets deleted.

        Returns:
            Callable[[~.RemoveDeploymentRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "remove_deployment" not in self._stubs:
            self._stubs["remove_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/RemoveDeployment",
                request_serializer=telcoautomation.RemoveDeploymentRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["remove_deployment"]

    @property
    def list_deployments(
        self,
    ) -> Callable[
        [telcoautomation.ListDeploymentsRequest],
        Awaitable[telcoautomation.ListDeploymentsResponse],
    ]:
        r"""Return a callable for the list deployments method over gRPC.

        List all deployments.

        Returns:
            Callable[[~.ListDeploymentsRequest],
                    Awaitable[~.ListDeploymentsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_deployments" not in self._stubs:
            self._stubs["list_deployments"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListDeployments",
                request_serializer=telcoautomation.ListDeploymentsRequest.serialize,
                response_deserializer=telcoautomation.ListDeploymentsResponse.deserialize,
            )
        return self._stubs["list_deployments"]

    @property
    def list_deployment_revisions(
        self,
    ) -> Callable[
        [telcoautomation.ListDeploymentRevisionsRequest],
        Awaitable[telcoautomation.ListDeploymentRevisionsResponse],
    ]:
        r"""Return a callable for the list deployment revisions method over gRPC.

        List deployment revisions of a given deployment.

        Returns:
            Callable[[~.ListDeploymentRevisionsRequest],
                    Awaitable[~.ListDeploymentRevisionsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_deployment_revisions" not in self._stubs:
            self._stubs["list_deployment_revisions"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListDeploymentRevisions",
                request_serializer=telcoautomation.ListDeploymentRevisionsRequest.serialize,
                response_deserializer=telcoautomation.ListDeploymentRevisionsResponse.deserialize,
            )
        return self._stubs["list_deployment_revisions"]

    @property
    def discard_deployment_changes(
        self,
    ) -> Callable[
        [telcoautomation.DiscardDeploymentChangesRequest],
        Awaitable[telcoautomation.DiscardDeploymentChangesResponse],
    ]:
        r"""Return a callable for the discard deployment changes method over gRPC.

        Discards the changes in a deployment and reverts the
        deployment to the last approved deployment revision. No
        changes take place if a deployment does not have
        revisions.

        Returns:
            Callable[[~.DiscardDeploymentChangesRequest],
                    Awaitable[~.DiscardDeploymentChangesResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "discard_deployment_changes" not in self._stubs:
            self._stubs[
                "discard_deployment_changes"
            ] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/DiscardDeploymentChanges",
                request_serializer=telcoautomation.DiscardDeploymentChangesRequest.serialize,
                response_deserializer=telcoautomation.DiscardDeploymentChangesResponse.deserialize,
            )
        return self._stubs["discard_deployment_changes"]

    @property
    def apply_deployment(
        self,
    ) -> Callable[
        [telcoautomation.ApplyDeploymentRequest], Awaitable[telcoautomation.Deployment]
    ]:
        r"""Return a callable for the apply deployment method over gRPC.

        Applies the deployment's YAML files to the parent
        orchestration cluster.

        Returns:
            Callable[[~.ApplyDeploymentRequest],
                    Awaitable[~.Deployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "apply_deployment" not in self._stubs:
            self._stubs["apply_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ApplyDeployment",
                request_serializer=telcoautomation.ApplyDeploymentRequest.serialize,
                response_deserializer=telcoautomation.Deployment.deserialize,
            )
        return self._stubs["apply_deployment"]

    @property
    def compute_deployment_status(
        self,
    ) -> Callable[
        [telcoautomation.ComputeDeploymentStatusRequest],
        Awaitable[telcoautomation.ComputeDeploymentStatusResponse],
    ]:
        r"""Return a callable for the compute deployment status method over gRPC.

        Returns the requested deployment status.

        Returns:
            Callable[[~.ComputeDeploymentStatusRequest],
                    Awaitable[~.ComputeDeploymentStatusResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "compute_deployment_status" not in self._stubs:
            self._stubs["compute_deployment_status"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ComputeDeploymentStatus",
                request_serializer=telcoautomation.ComputeDeploymentStatusRequest.serialize,
                response_deserializer=telcoautomation.ComputeDeploymentStatusResponse.deserialize,
            )
        return self._stubs["compute_deployment_status"]

    @property
    def rollback_deployment(
        self,
    ) -> Callable[
        [telcoautomation.RollbackDeploymentRequest],
        Awaitable[telcoautomation.Deployment],
    ]:
        r"""Return a callable for the rollback deployment method over gRPC.

        Rollback the active deployment to the given past
        approved deployment revision.

        Returns:
            Callable[[~.RollbackDeploymentRequest],
                    Awaitable[~.Deployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "rollback_deployment" not in self._stubs:
            self._stubs["rollback_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/RollbackDeployment",
                request_serializer=telcoautomation.RollbackDeploymentRequest.serialize,
                response_deserializer=telcoautomation.Deployment.deserialize,
            )
        return self._stubs["rollback_deployment"]

    @property
    def get_hydrated_deployment(
        self,
    ) -> Callable[
        [telcoautomation.GetHydratedDeploymentRequest],
        Awaitable[telcoautomation.HydratedDeployment],
    ]:
        r"""Return a callable for the get hydrated deployment method over gRPC.

        Returns the requested hydrated deployment.

        Returns:
            Callable[[~.GetHydratedDeploymentRequest],
                    Awaitable[~.HydratedDeployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_hydrated_deployment" not in self._stubs:
            self._stubs["get_hydrated_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/GetHydratedDeployment",
                request_serializer=telcoautomation.GetHydratedDeploymentRequest.serialize,
                response_deserializer=telcoautomation.HydratedDeployment.deserialize,
            )
        return self._stubs["get_hydrated_deployment"]

    @property
    def list_hydrated_deployments(
        self,
    ) -> Callable[
        [telcoautomation.ListHydratedDeploymentsRequest],
        Awaitable[telcoautomation.ListHydratedDeploymentsResponse],
    ]:
        r"""Return a callable for the list hydrated deployments method over gRPC.

        List all hydrated deployments present under a
        deployment.

        Returns:
            Callable[[~.ListHydratedDeploymentsRequest],
                    Awaitable[~.ListHydratedDeploymentsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_hydrated_deployments" not in self._stubs:
            self._stubs["list_hydrated_deployments"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ListHydratedDeployments",
                request_serializer=telcoautomation.ListHydratedDeploymentsRequest.serialize,
                response_deserializer=telcoautomation.ListHydratedDeploymentsResponse.deserialize,
            )
        return self._stubs["list_hydrated_deployments"]

    @property
    def update_hydrated_deployment(
        self,
    ) -> Callable[
        [telcoautomation.UpdateHydratedDeploymentRequest],
        Awaitable[telcoautomation.HydratedDeployment],
    ]:
        r"""Return a callable for the update hydrated deployment method over gRPC.

        Updates a hydrated deployment.

        Returns:
            Callable[[~.UpdateHydratedDeploymentRequest],
                    Awaitable[~.HydratedDeployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_hydrated_deployment" not in self._stubs:
            self._stubs[
                "update_hydrated_deployment"
            ] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/UpdateHydratedDeployment",
                request_serializer=telcoautomation.UpdateHydratedDeploymentRequest.serialize,
                response_deserializer=telcoautomation.HydratedDeployment.deserialize,
            )
        return self._stubs["update_hydrated_deployment"]

    @property
    def apply_hydrated_deployment(
        self,
    ) -> Callable[
        [telcoautomation.ApplyHydratedDeploymentRequest],
        Awaitable[telcoautomation.HydratedDeployment],
    ]:
        r"""Return a callable for the apply hydrated deployment method over gRPC.

        Applies a hydrated deployment to a workload cluster.

        Returns:
            Callable[[~.ApplyHydratedDeploymentRequest],
                    Awaitable[~.HydratedDeployment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "apply_hydrated_deployment" not in self._stubs:
            self._stubs["apply_hydrated_deployment"] = self._logged_channel.unary_unary(
                "/google.cloud.telcoautomation.v1.TelcoAutomation/ApplyHydratedDeployment",
                request_serializer=telcoautomation.ApplyHydratedDeploymentRequest.serialize,
                response_deserializer=telcoautomation.HydratedDeployment.deserialize,
            )
        return self._stubs["apply_hydrated_deployment"]

    def _prep_wrapped_messages(self, client_info):
        """Precompute the wrapped methods, overriding the base class method to use async wrappers."""
        self._wrapped_methods = {
            self.list_orchestration_clusters: self._wrap_method(
                self.list_orchestration_clusters,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_orchestration_cluster: self._wrap_method(
                self.get_orchestration_cluster,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_orchestration_cluster: self._wrap_method(
                self.create_orchestration_cluster,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_orchestration_cluster: self._wrap_method(
                self.delete_orchestration_cluster,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_edge_slms: self._wrap_method(
                self.list_edge_slms,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_edge_slm: self._wrap_method(
                self.get_edge_slm,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_edge_slm: self._wrap_method(
                self.create_edge_slm,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_edge_slm: self._wrap_method(
                self.delete_edge_slm,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_blueprint: self._wrap_method(
                self.create_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_blueprint: self._wrap_method(
                self.update_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_blueprint: self._wrap_method(
                self.get_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_blueprint: self._wrap_method(
                self.delete_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_blueprints: self._wrap_method(
                self.list_blueprints,
                default_timeout=None,
                client_info=client_info,
            ),
            self.approve_blueprint: self._wrap_method(
                self.approve_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.propose_blueprint: self._wrap_method(
                self.propose_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.reject_blueprint: self._wrap_method(
                self.reject_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_blueprint_revisions: self._wrap_method(
                self.list_blueprint_revisions,
                default_timeout=None,
                client_info=client_info,
            ),
            self.search_blueprint_revisions: self._wrap_method(
                self.search_blueprint_revisions,
                default_timeout=None,
                client_info=client_info,
            ),
            self.search_deployment_revisions: self._wrap_method(
                self.search_deployment_revisions,
                default_timeout=None,
                client_info=client_info,
            ),
            self.discard_blueprint_changes: self._wrap_method(
                self.discard_blueprint_changes,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_public_blueprints: self._wrap_method(
                self.list_public_blueprints,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_public_blueprint: self._wrap_method(
                self.get_public_blueprint,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_deployment: self._wrap_method(
                self.create_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_deployment: self._wrap_method(
                self.update_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_deployment: self._wrap_method(
                self.get_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.remove_deployment: self._wrap_method(
                self.remove_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_deployments: self._wrap_method(
                self.list_deployments,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_deployment_revisions: self._wrap_method(
                self.list_deployment_revisions,
                default_timeout=None,
                client_info=client_info,
            ),
            self.discard_deployment_changes: self._wrap_method(
                self.discard_deployment_changes,
                default_timeout=None,
                client_info=client_info,
            ),
            self.apply_deployment: self._wrap_method(
                self.apply_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.compute_deployment_status: self._wrap_method(
                self.compute_deployment_status,
                default_timeout=None,
                client_info=client_info,
            ),
            self.rollback_deployment: self._wrap_method(
                self.rollback_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_hydrated_deployment: self._wrap_method(
                self.get_hydrated_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_hydrated_deployments: self._wrap_method(
                self.list_hydrated_deployments,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_hydrated_deployment: self._wrap_method(
                self.update_hydrated_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.apply_hydrated_deployment: self._wrap_method(
                self.apply_hydrated_deployment,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_location: self._wrap_method(
                self.get_location,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_locations: self._wrap_method(
                self.list_locations,
                default_timeout=None,
                client_info=client_info,
            ),
            self.cancel_operation: self._wrap_method(
                self.cancel_operation,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_operation: self._wrap_method(
                self.delete_operation,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_operation: self._wrap_method(
                self.get_operation,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_operations: self._wrap_method(
                self.list_operations,
                default_timeout=None,
                client_info=client_info,
            ),
        }

    def _wrap_method(self, func, *args, **kwargs):
        if self._wrap_with_kind:  # pragma: NO COVER
            kwargs["kind"] = self.kind
        return gapic_v1.method_async.wrap_method(func, *args, **kwargs)

    def close(self):
        return self._logged_channel.close()

    @property
    def kind(self) -> str:
        return "grpc_asyncio"

    @property
    def delete_operation(
        self,
    ) -> Callable[[operations_pb2.DeleteOperationRequest], None]:
        r"""Return a callable for the delete_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_operation" not in self._stubs:
            self._stubs["delete_operation"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/DeleteOperation",
                request_serializer=operations_pb2.DeleteOperationRequest.SerializeToString,
                response_deserializer=None,
            )
        return self._stubs["delete_operation"]

    @property
    def cancel_operation(
        self,
    ) -> Callable[[operations_pb2.CancelOperationRequest], None]:
        r"""Return a callable for the cancel_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "cancel_operation" not in self._stubs:
            self._stubs["cancel_operation"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/CancelOperation",
                request_serializer=operations_pb2.CancelOperationRequest.SerializeToString,
                response_deserializer=None,
            )
        return self._stubs["cancel_operation"]

    @property
    def get_operation(
        self,
    ) -> Callable[[operations_pb2.GetOperationRequest], operations_pb2.Operation]:
        r"""Return a callable for the get_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_operation" not in self._stubs:
            self._stubs["get_operation"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/GetOperation",
                request_serializer=operations_pb2.GetOperationRequest.SerializeToString,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["get_operation"]

    @property
    def list_operations(
        self,
    ) -> Callable[
        [operations_pb2.ListOperationsRequest], operations_pb2.ListOperationsResponse
    ]:
        r"""Return a callable for the list_operations method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_operations" not in self._stubs:
            self._stubs["list_operations"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/ListOperations",
                request_serializer=operations_pb2.ListOperationsRequest.SerializeToString,
                response_deserializer=operations_pb2.ListOperationsResponse.FromString,
            )
        return self._stubs["list_operations"]

    @property
    def list_locations(
        self,
    ) -> Callable[
        [locations_pb2.ListLocationsRequest], locations_pb2.ListLocationsResponse
    ]:
        r"""Return a callable for the list locations method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_locations" not in self._stubs:
            self._stubs["list_locations"] = self._logged_channel.unary_unary(
                "/google.cloud.location.Locations/ListLocations",
                request_serializer=locations_pb2.ListLocationsRequest.SerializeToString,
                response_deserializer=locations_pb2.ListLocationsResponse.FromString,
            )
        return self._stubs["list_locations"]

    @property
    def get_location(
        self,
    ) -> Callable[[locations_pb2.GetLocationRequest], locations_pb2.Location]:
        r"""Return a callable for the list locations method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_location" not in self._stubs:
            self._stubs["get_location"] = self._logged_channel.unary_unary(
                "/google.cloud.location.Locations/GetLocation",
                request_serializer=locations_pb2.GetLocationRequest.SerializeToString,
                response_deserializer=locations_pb2.Location.FromString,
            )
        return self._stubs["get_location"]


__all__ = ("TelcoAutomationGrpcAsyncIOTransport",)
